/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.kenai.com
 * SCM: https://kenai.com/hg/openbluesky~src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.capabilitiesprovider;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import it.tidalwave.util.Task;
import org.openide.util.Lookup;
import lombok.AccessLevel;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

/***********************************************************************************************************************
 * 
 * A class which manages thread-scope injection of capabilities. It provides a method that binds a Task to a Lookup
 * and arranges things so that all PresentationModels created within the task can have extra capabilities injected from
 * the Lookup. FIXME: this should work with any AsLookupSuppor subclass, not only PresentationModel.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@RequiredArgsConstructor(access=AccessLevel.PRIVATE) @Slf4j
public class ThreadLookupBinder 
  {
    private final Lookup lookup;

    private static final ThreadLocal<Lookup> LOOKUP_THREAD_LOCAL = new ThreadLocal<Lookup>();
    
    @CheckForNull
    public static Lookup getLookup()
      {
        return LOOKUP_THREAD_LOCAL.get();
      }
    
    @Nonnull
    public static ThreadLookupBinder with (final @Nonnull Lookup lookup)
      {
        return new ThreadLookupBinder(lookup);  
      }
    
    @Nonnull
    public static ThreadLookupBinder with (final @Nonnull Lookup.Provider lookupProvider)
      {
        return new ThreadLookupBinder(lookupProvider.getLookup());  
      }
    
    public <T, E extends Throwable> T run (final @Nonnull Task<T, E> task) 
      throws E
      {
        try
          {
            LOOKUP_THREAD_LOCAL.set(lookup);
            log.trace("Bound {} to thread {}", lookup, Thread.currentThread());
            return task.run();
          }
        finally
          {
            LOOKUP_THREAD_LOCAL.remove();
            log.trace("Unbound {} to thread {}", lookup, Thread.currentThread());
          }
      }
  }
